//-------------------------------------------------------------------------------
//$Header: /home/dashley/cvsrep/uculib01/uculib01/src/stm8/cosmic/modxx/atatanix100odegx1rrxx/devel/test_trace.c,v 1.2 2010/02/20 03:36:05 dashley Exp $
//-------------------------------------------------------------------------------
//Copyright (c)2010 David T. Ashley
//
//Permission is hereby granted, free of charge, to any person obtaining a copy
//of this software source code and associated documentation files (the
//"Software"), to deal in the Software without restriction, including without
//limitation the rights to use, copy, modify, merge, publish, distribute,
//sublicense, and/or sell copies of the Software, and to permit persons to whom
//the Software is furnished to do so, subject to the following conditions:
//
//The above copyright notice and this permission notice shall be included in
//all copies or substantial portions of the Software.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//THE SOFTWARE.
//-------------------------------------------------------------------------------
//The purpose of this program is to simulate and verify the behavior of the 
//UcuAtAtanIx100Odegx1RRxx() function.  This program is designed to run on 
//an x86 *nix box.
//-------------------------------------------------------------------------------
#include <stdio.h>

typedef unsigned char     UCU_UINT8;
typedef unsigned short    UCU_UINT16;
typedef unsigned int      UCU_UINT32;


UCU_UINT8 UcuAtAtanIx100Odegx1RRxx(UCU_UINT16 arg)
{
   static const UCU_UINT16 atan_lu[90] =  // arctan() precision of 1 degree
   {
   //This table was generated by a C program.
   //The integers below represent the "breakpoints".  For example,
   //the tangents of 48 and 49 degrees are:
   //  48 :  1.1106
   //  49 :  1.1504
   //The entry under 48 below (113) does not represent the tangent of 48 
   //degrees.  Rather, it represents the point at which the returned arctangent
   //should switch from 48 to 49 degrees.  The table entries are approximately
   //the midpoints between arctangents of integer degrees.  This allows the
   //returned arctangent to be to the closest degree.
   //
        /*  0 degrees */       0,     2,     4,     6,     7,
        /*  5 degrees */       9,    11,    13,    14,    16,
        /* 10 degrees */      18,    20,    22,    24,    25,
        /* 15 degrees */      27,    29,    31,    33,    35,
        /* 20 degrees */      37,    39,    41,    43,    45,
        /* 25 degrees */      47,    49,    52,    54,    56,
        /* 30 degrees */      58,    61,    63,    66,    68,
        /* 35 degrees */      71,    73,    76,    79,    82,
        /* 40 degrees */      85,    88,    91,    94,    98,
        /* 45 degrees */     101,   105,   109,   113,   117,
        /* 50 degrees */     121,   125,   130,   135,   140,
        /* 55 degrees */     145,   151,   156,   163,   169,
        /* 60 degrees */     176,   184,   192,   200,   209,
        /* 65 degrees */     219,   229,   241,   253,   267,
        /* 70 degrees */     282,   298,   317,   337,   360,
        /* 75 degrees */     386,   416,   451,   491,   539,
        /* 80 degrees */     597,   669,   759,   877,  1038,
        /* 85 degrees */    1270,  1634,  2290,  3818, 11458
   };

   UCU_UINT8 bottom, top, middle;

   printf("--------------------\n");
   printf("Input: %u\n", arg);

   bottom = 0;
   top    = 90;  //Note:  element [90] in the lookup table
                 //isn't actually necessary.  If 90 is the eventual
                 //return value, bottom converges to top and top
                 //remains unchanged.
                 //
                 //Once (bottom == top), no more dereferencing is done.
                 //This was verified both by hand analysis and by running
                 //the program on a *nix platform.

   //This algorithm is a modified classic binary search.
   //
   while (bottom != top)
   {
      middle = bottom + top;
      middle >>= 1;

      printf("   bottom/middle/top = %2u/%2u/%2u\n", bottom, middle, top);

      if (middle > 89)
   	 {
	 printf("Error\n");
	 printf("Error\n");
	 printf("Error\n");
	 printf("Error\n");
	 printf("Error\n");
	 printf("Error\n");
	 printf("Error\n");
	 printf("Error\n");
	 printf("Error\n");
	 printf("Error\n");
	 }

      if (atan_lu[middle] >= arg)
      {
         //The "middle" table entry is at least as large as the argument.
         //We are clear to lower the ceiling to be the middle.
         // 
         top = middle;
      }
      else
      {
         //The "middle" entry is less than the argument.  That automatically
         //makes the middle unsuitable, so the correct return value has
         //to be at least one larger than the middle.
         //
         bottom = middle + 1;
      }
   }

   printf("   returning: %u\n", bottom);

   return(bottom);
}


int main(void)
   {
   UCU_UINT32 i;
   UCU_UINT8  rv;

   for (i=0; i<65536; i++)
     {
     rv = UcuAtAtanIx100Odegx1RRxx(i);
     }

   printf("--------------------\n");

   return(0);
   }


//-------------------------------------------------------------------------------
//End of $Id: test_trace.c,v 1.2 2010/02/20 03:36:05 dashley Exp $
//-------------------------------------------------------------------------------
//$Log: test_trace.c,v $
//Revision 1.2  2010/02/20 03:36:05  dashley
//Edits.
//
//Revision 1.1  2010/02/20 02:38:08  dashley
//Initial checkin.
//-------------------------------------------------------------------------------
